@@ -2,6 +2,7 @@ from typing import Any, Iterable, List, NamedTuple, Optional, Union
 
 import htmlgenerator as hg
 from django.db import models
+from django.db.models.constants import LOOKUP_SEP
 from django.utils.translation import gettext_lazy as _
 
 from basxbread.utils import filter_fieldlist, resolve_modellookup
@@ -288,7 +289,7 @@ event.stopPropagation()""",
                         style="margin: 0; padding: 0",
                     ),
                     _class="settingscontainer",
-                    style="position: absolute; z-index: 999; right: 0; display: none",
+                    style="position: absolute; z-index: 999; right: 0; display: none; max-width: 80%",
                     onload="document.addEventListener('click', (e) => {this.style.display = 'none'})",
                 ),
                 style="position: relative",
@@ -306,6 +307,7 @@ event.stopPropagation()""",
         # column behaviour
         columns: Iterable[Union[str, "DataTableColumn"]] = (),
         prevent_automatic_sortingnames=False,
+        prevent_automatic_filternames=False,
         # row behaviour
         rowvariable="row",
         rowactions: Iterable[Link] = (),
@@ -400,12 +402,12 @@ event.stopPropagation()""",
                 }
             # convert simple string (modelfield) to column definition
             if isinstance(col, str):
-
                 col = DataTableColumn.from_modelfield(
-                    col,
-                    model,
-                    prevent_automatic_sortingnames,
-                    rowvariable,
+                    col=col,
+                    model=model,
+                    prevent_automatic_sortingnames=prevent_automatic_sortingnames,
+                    prevent_automatic_filternames=prevent_automatic_filternames,
+                    rowvariable=rowvariable,
                     td_attributes=td_attributes,
                 )
             else:
@@ -488,6 +490,9 @@ event.stopPropagation()""",
         return classes
 
 
+# Helpers for building columns ###################################################################
+
+
 class DataTableColumn(NamedTuple):
     header: Any
     cell: Any
@@ -495,6 +500,7 @@ class DataTableColumn(NamedTuple):
     enable_row_click: bool = True
     th_attributes: Optional[Union[hg.F, dict]] = None
     td_attributes: Optional[Union[hg.F, dict]] = None
+    filtername: Optional[str] = None
 
     @staticmethod
     def from_modelfield(
@@ -504,13 +510,17 @@ class DataTableColumn(NamedTuple):
         rowvariable="row",
         th_attributes=None,
         td_attributes=None,
+        prevent_automatic_filternames=False,
     ) -> "DataTableColumn":
         return DataTableColumn(
-            ObjectFieldLabel(col, model),
-            ObjectFieldValue(col, rowvariable),
-            sortingname_for_column(model, col)
+            header=ObjectFieldLabel(col, model),
+            cell=ObjectFieldValue(col, rowvariable),
+            sortingname=sortingname_for_column(model, col)
             if not prevent_automatic_sortingnames
             else None,
+            filtername=filtername_for_column(model, col)
+            if not prevent_automatic_filternames
+            else None,
             th_attributes=th_attributes,
             td_attributes=td_attributes,
         )
@@ -536,6 +546,64 @@ class DataTableColumn(NamedTuple):
         return hg.TH(headcontent, lazy_attributes=self.th_attributes)
 
 
+def sortingclass_for_column(orderingurlparameter, columnname):
+    def extracturlparameter(context):
+        value = context["request"].GET.get(orderingurlparameter, "")
+        if not value:
+            return ""
+        if value == columnname:
+            return "bx--table-sort--active"
+        if value == "-" + columnname:
+            return "bx--table-sort--active bx--table-sort--ascending"
+        return ""
+
+    return hg.F(extracturlparameter)
+
+
+def sortingname_for_column(model, column):
+    components = []
+    for field in resolve_modellookup(model, column):
+        if hasattr(field, "sorting_name"):
+            components.append(field.sorting_name)
+        elif isinstance(field, (models.Field, models.ForeignObjectRel)):
+            components.append(field.name)
+        else:
+            return None
+    return LOOKUP_SEP.join(components)
+
+
+def filtername_for_column(model, column):
+    components = []
+    for field in resolve_modellookup(model, column):
+        if isinstance(field, (models.Field, models.ForeignObjectRel)):
+            components.append(field.name)
+    return LOOKUP_SEP.join(components)
+
+
+def sortinglink_for_column(orderingurlparameter, columnname):
+    ORDERING_VALUES = {
+        None: columnname,
+        columnname: "-" + columnname,
+        "-" + columnname: None,
+    }
+
+    def extractsortinglink(context):
+        currentordering = context["request"].GET.get(orderingurlparameter, None)
+        nextordering = ORDERING_VALUES.get(currentordering, columnname)
+        ret = link_with_urlparameters(
+            context["request"], **{orderingurlparameter: nextordering}
+        )
+        # workaround to allow reseting session-stored table state
+        if nextordering is None and "?" not in ret:
+            ret = ret + "?reset"
+        return ret
+
+    return aslink_attributes(hg.F(extractsortinglink))
+
+
+# Helper for building a searchbar ##############################################
+
+
 def searchbar(search_urlparameter: str):
     """
     Creates a searchbar element for datatables to submit an entered search
@@ -577,50 +645,3 @@ def searchbar(search_urlparameter: str):
         ),
         _class="bx--toolbar-search-container-persistent",
     )
-
-
-def sortingclass_for_column(orderingurlparameter, columnname):
-    def extracturlparameter(context):
-        value = context["request"].GET.get(orderingurlparameter, "")
-        if not value:
-            return ""
-        if value == columnname:
-            return "bx--table-sort--active"
-        if value == "-" + columnname:
-            return "bx--table-sort--active bx--table-sort--ascending"
-        return ""
-
-    return hg.F(extracturlparameter)
-
-
-def sortingname_for_column(model, column):
-    components = []
-    for field in resolve_modellookup(model, column):
-        if hasattr(field, "sorting_name"):
-            components.append(field.sorting_name)
-        elif isinstance(field, (models.Field, models.ForeignObjectRel)):
-            components.append(field.name)
-        else:
-            return None
-    return "__".join(components)
-
-
-def sortinglink_for_column(orderingurlparameter, columnname):
-    ORDERING_VALUES = {
-        None: columnname,
-        columnname: "-" + columnname,
-        "-" + columnname: None,
-    }
-
-    def extractsortinglink(context):
-        currentordering = context["request"].GET.get(orderingurlparameter, None)
-        nextordering = ORDERING_VALUES.get(currentordering, columnname)
-        ret = link_with_urlparameters(
-            context["request"], **{orderingurlparameter: nextordering}
-        )
-        # workaround to allow reseting session-stored table state
-        if nextordering is None and "?" not in ret:
-            ret = ret + "?reset"
-        return ret
-
-    return aslink_attributes(hg.F(extractsortinglink))
