@@ -1,13 +1,18 @@
 from typing import Callable, Iterable, List, NamedTuple, Optional, Union
 
+import django_filters
 import htmlgenerator as hg
+from django import forms
 from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin
+from django.core.exceptions import FieldDoesNotExist
 from django.db import models
+from django.db.models.constants import LOOKUP_SEP
 from django.http import HttpRequest, HttpResponse
 from django.shortcuts import redirect
 from django.utils.translation import gettext_lazy as _
+from django.utils.translation import pgettext_lazy
 from django.views.generic import ListView
 from djangoql.exceptions import DjangoQLError
 from djangoql.queryset import apply_search
@@ -136,8 +141,9 @@ class BrowseView(BaseView, LoginRequiredMixin, PermissionListMixin, ListView):
         )
         self.title = kwargs.get("title") or self.title
         self.rowactions = kwargs.get("rowactions") or self.rowactions
+        self.model = kwargs.get("model") or self.model
         self.columns = expand_ALL_constant(
-            kwargs.get("model") or self.model, kwargs.get("columns") or self.columns
+            self.model, kwargs.get("columns") or self.columns
         )
         self.rowclickaction = kwargs.get("rowclickaction") or self.rowclickaction
         self.backurl = kwargs.get("backurl") or self.backurl
@@ -145,6 +151,16 @@ class BrowseView(BaseView, LoginRequiredMixin, PermissionListMixin, ListView):
         self.viewstate_sessionkey = (
             kwargs.get("viewstate_sessionkey") or self.viewstate_sessionkey
         )
+        self.filterset_class = parse_filterconfig(
+            self.model,
+            ("AND",)
+            + tuple(
+                get_filtername(column)
+                for column in self.columns
+                if get_filtername(column)
+            ),
+            prefix="f",
+        )
         super().__init__(*args, **kwargs)
         self.bulkactions = (
             kwargs.get("bulkactions")
@@ -172,8 +188,10 @@ class BrowseView(BaseView, LoginRequiredMixin, PermissionListMixin, ListView):
         paginate_by = self.get_paginate_by(qs)
         if paginate_by is None:
             paginate_by = qs.count()
+        qs = self.paginate_queryset(qs, paginate_by)[2] if paginate_by > 0 else qs
+
         return layout.datatable.DataTable.from_queryset(
-            self.paginate_queryset(qs, paginate_by)[2] if paginate_by > 0 else qs,
+            queryset=qs,
             columns=self.columns,
             bulkactions=bulkactions,
             rowactions=self.rowactions,
@@ -205,7 +223,9 @@ class BrowseView(BaseView, LoginRequiredMixin, PermissionListMixin, ListView):
         }
 
     def get_settingspanel(self):
-        return None
+        return build_filterpanel(
+            self.filterset_class(self.request.GET, queryset=self.get_queryset())
+        )
 
     def get_required_permissions(self, request):
         return [f"{self.model._meta.app_label}.view_{self.model.__name__.lower()}"]
@@ -287,6 +307,8 @@ class BrowseView(BaseView, LoginRequiredMixin, PermissionListMixin, ListView):
                     )
                 )
 
+        qs = self.filterset_class(self.request.GET, queryset=qs).qs
+
         selectedobjects = self.request.GET.getlist(self.objectids_urlparameter)
         if selectedobjects and "all" not in selectedobjects:
             qs &= super().get_queryset().filter(pk__in=selectedobjects)
@@ -441,3 +463,225 @@ def restore(request, queryset, softdeletefield, required_permissions=None):
             else queryset.model._meta.verbose_name,
         },
     )
+
+
+class AndGroup(django_filters.FilterSet):
+    prefix = None
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.subgroups = []
+        for group in self.subgroup_classes:
+            self.subgroups.append(group(*args, **kwargs))
+        self.form_prefix = self.prefix
+
+    def filter_queryset(self, queryset):
+        for name, value in self.form.cleaned_data.items():
+            queryset = self.filters[name].filter(queryset, value)
+            assert isinstance(
+                queryset, models.QuerySet
+            ), "Expected '%s.%s' to return a QuerySet, but got a %s instead." % (
+                type(self).__name__,
+                name,
+                type(queryset).__name__,
+            )
+
+        for group in self.subgroups:
+            queryset = group.filter_queryset(queryset)
+
+        return queryset
+
+
+class OrGroup(django_filters.FilterSet):
+    prefix = None
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.subgroups = []
+        for group in self.subgroup_classes:
+            self.subgroups.append(group(*args, **kwargs))
+        self.form_prefix = self.prefix
+
+    def filter_queryset(self, queryset):
+        for name, value in self.form.cleaned_data.items():
+            queryset |= self.filters[name].filter(queryset, value)
+            assert isinstance(
+                queryset, models.QuerySet
+            ), "Expected '%s.%s' to return a QuerySet, but got a %s instead." % (
+                type(self).__name__,
+                name,
+                type(queryset).__name__,
+            )
+        for group in self.subgroups:
+            queryset |= group.filter_queryset(queryset)
+        return queryset
+
+
+def parse_filterconfig(basemodel, filterconfig, prefix):
+    """
+    filterconfig: Tree in the form of
+                  ("and"
+                      ("or", fieldname1, fieldname2),
+                      ("or", fieldname3, fieldname4),
+                  )
+    """
+    FILTERSETTYPE = {"and": AndGroup, "or": OrGroup}
+    grouptype, *subfields = filterconfig
+    if grouptype.lower() not in FILTERSETTYPE:
+        raise ValueError(
+            f"Invalid filter group specified '{filterconfig[0]}', must be 'and' or 'or'"
+        )
+
+    fields = []
+    subgroups = []
+    n = 0
+    for f in subfields:
+        if isinstance(f, str):
+            try:
+                modelfield = get_field(basemodel, f)
+            except FieldDoesNotExist:
+                continue
+            # ignore filefields
+            if not isinstance(modelfield, (models.FileField)):
+                fields.append(f)
+        elif isinstance(f, Iterable):
+            subgroups.append(parse_filterconfig(basemodel, f, prefix + str(n)))
+            n += 1
+        else:
+            raise ValueError(
+                f"Declared filter field '{f}' is not of type {(str, OrGroup, AndGroup)} but {type(f)}"
+            )
+
+    meta = type(
+        "Meta",
+        (),
+        {
+            "model": basemodel,
+            "fields": fields,
+            "filter_overrides": FILTER_OVERRIDES,
+        },
+    )
+
+    ret = type(
+        f"{type(basemodel).__name__}FilterSet",
+        (FILTERSETTYPE[grouptype.lower()],),
+        {"Meta": meta, "subgroup_classes": subgroups, "prefix": f"filter_{prefix}"},
+    )
+    return ret
+
+
+def get_filtername(column):
+    if isinstance(column, str):
+        return column.replace(".", LOOKUP_SEP)
+    if isinstance(column, layout.datatable.DataTableColumn):
+        return column.filtername
+    return None
+
+
+def get_field(basemodel, fieldname):
+    if LOOKUP_SEP in fieldname:
+        relatedfield, rest = fieldname.split(LOOKUP_SEP, 1)
+        return get_field(basemodel._meta.get_field(relatedfield).related_model, rest)
+    return basemodel._meta.get_field(fieldname)
+
+
+def build_filterpanel(filterset):
+    return hg.DIV(
+        hg.DIV(
+            hg.DIV(
+                hg.DIV(_("Filter"), style="margin-bottom: 1rem"),
+                hg.DIV(
+                    layout.components.forms.Form(
+                        forms.Form(),
+                        _build_filter_ui_recursive(filterset),
+                        _class="filterform",
+                        method="GET",
+                    ),
+                    style="display: flex",
+                ),
+            ),
+            style="display: flex; padding: 24px 32px 0 32px",
+        ),
+        hg.DIV(
+            layout.button.Button(
+                _("Cancel"),
+                buttontype="ghost",
+                onclick="this.closest('.settingscontainer').style.display = 'none'",
+            ),
+            layout.button.Button.from_link(
+                Link(
+                    label=_("Reset"),
+                    href=hg.format("{}?reset=1", hg.C("request").path),
+                    iconname=None,
+                ),
+                buttontype="secondary",
+            ),
+            layout.button.Button(
+                pgettext_lazy("apply filter", "Filter"),
+                onclick="""this.closest('.filterpanel').querySelector('.filterform').submit();""",
+            ),
+            style="display: flex; justify-content: flex-end; margin-top: 24px",
+            _class="bx--modal-footer",
+        ),
+        _class="filterpanel",
+        style="background-color: #fff",
+    )
+
+
+def _build_filter_ui_recursive(filterset):
+    flexdir = ""
+    if isinstance(filterset, AndGroup):
+        flexdir = "column"
+    elif isinstance(filterset, OrGroup):
+        flexdir = "row"
+
+    return hg.DIV(
+        layout.components.forms.Form(
+            filterset.form,
+            *[
+                layout.components.forms.FormField(
+                    f, no_wrapper=True, style="margin-bottom: 1rem; margin-right: 1rem"
+                )
+                for f in filterset.form.fields
+            ],
+            standalone=False,
+            *[_build_filter_ui_recursive(f) for f in filterset.subgroups],
+        ),
+        style="display: flex; flex-direction: " + flexdir,
+    )
+
+
+FILTER_OVERRIDES = {
+    models.CharField: {
+        "filter_class": django_filters.CharFilter,
+        "extra": lambda f: {"lookup_expr": "icontains"},
+    },
+    models.TextField: {
+        "filter_class": django_filters.CharFilter,
+        "extra": lambda f: {"lookup_expr": "icontains"},
+    },
+    models.EmailField: {
+        "filter_class": django_filters.CharFilter,
+        "extra": lambda f: {"lookup_expr": "icontains"},
+    },
+    models.URLField: {
+        "filter_class": django_filters.CharFilter,
+        "extra": lambda f: {"lookup_expr": "icontains"},
+    },
+    models.DateField: {
+        "filter_class": django_filters.DateFromToRangeFilter,
+        "extra": lambda f: {
+            "widget": django_filters.widgets.DateRangeWidget(
+                attrs={"type": "text", "class": "validate datepicker"}
+            )
+        },
+    },
+    models.DateTimeField: {
+        "filter_class": django_filters.DateFromToRangeFilter,
+        "extra": lambda f: {
+            "widget": django_filters.widgets.DateRangeWidget(
+                attrs={"type": "text", "class": "validate datepicker"}
+            )
+        },
+    },
+}
